% This code solves a bewley model, then projects both implicit and explicit
% partitions, write DYNARE code and run the files.
% If your run it, it should work (if simul_eco.mat is in the folder),
% otherwise you have to use solve_steady.m by setting ind_check = true below 


% It is based on two Objects and a file
%    1) "bewley" is the Bewley economy both parameter values and equilibrium
%    allocation
%    2) "proj" is the projected model. it can be either an implicit or
%    explicit partition
%    3) "Dynare_simul.mod" is the dynare file, which can be run
%
%
% The objects are constructed by four procedures:
%     1) [bewley,distr] = solve_steady(bewley,r,maxiter,maxV,display);
%     solves the
%     steady_state of the Bewley economy and stores results in the "bewley"
%     object:
%           - bewley must contain relevant parameters (defined in Section 1
%           below)
%           - r is the initial interest rate for iteration
%           - maxwiter is the maximum iteration on "r" before stopping
%           convergence procedure (set it to 1 if you want partial equilibrium with an interest rater) 
%           - maxV is the maximum iteration for the value finction
%           iteration
%           - display can either "on" or "off" to exhibit iteration steps
%
%     2)  proj = projection_explicit(bewley,N);
%       This procedure constructs the explicit projection with a truncation
%       after N periods and store the result in proj
%
%     3) proj = projection_implicit(bewley,N);
%       This procedure constructs the implicit projection with an intended number of bin N 
%        The actual number of bin can be different due to some rounding of integers.
%
%     4) write_dynare_simul(bewley,proj,periods);
%           write the "Dynare_simul.mod" for the projected model (with
%           either implicit or explicit partitions). periods is the number
%           of periods to simulate the model. if it is 0, then theoretical
%           moments are provided.

clc; clear; close all
ind_check = false;          % set it to true if you have to find the steady state (it is stored in "simul_eco.mat")

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    1)  Parameter values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


bewley.b        = 0.1;       % UI Benefit
bewley.bet      = 0.99;      % discount factor
bewley.sigma    = 1;         % concavity utility function
bewley.alpha    = 0.36;      % Capital Share
bewley.delta    = 0.025;     % Capital Depreciation
bewley.phil     = .0000001;  % Frish elasticity of the labor supply
bewley.xsil     = 1;         % scaling parameter of labor supply
bewley.lbar     = .1 ;       % scaling parameter for unemployed agents


bewley.nx       = 1;         % Number of productivity level (1 in the simulation case, for a Krusell-Smith economy)

Urec            = 0.07;      % Average unemployment rate
meanL           = 1;         % Working time
bewley.Urec     = Urec;
bewley.rho      = (0.9695)^(1/4); %Persistence component of income
rho             = bewley.rho;
sigy            = sqrt(0.0384/(1+sqrt(rho)+rho+rho^(3/2)));

%%% Productivity grid
if bewley.nx>1
    [bewley.ex,bewley.piex] = rouwenhorst(bewley.nx,0,rho,sigy); % transition matrix across productivity
    temp         = bewley.piex^1000000;
    bewley.exinv = temp(1,:);    
    bewley.ex    = exp(bewley.ex);
    bewley.ex    = bewley.ex/(bewley.exinv*bewley.ex);
    bewley.e(1)  = bewley.b*bewley.lbar;
    bewley.e(2)  = 1*bewley.lbar;
else
    bewley.ex    = 1;
    bewley.piex  = 1;
    bewley.exinv = 1;
end;
    
       
%%% Transition matrix on the labor market 
bewley.ne        = 2;
FRR              = 0.5;
bewley.pie(1,2)  = FRR;
bewley.pie(1,1)  = 1-bewley.pie(1,2);
bewley.pie(2,1)  = (1-bewley.pie(1,1))*Urec/(1-Urec);
bewley.pie(2,2)  = 1-bewley.pie(2,1);


%kgrid construction (for the capital)
kmin            = 0; 
kmax            = 250; 
knum            = 1000; 
curv            = 2;        %curvature of the exponential grid
bewley.curv     = curv;
bewley.knum     = knum;
kgrid           = kmin+(kmax-kmin)*(linspace(0,1,knum).^curv)';
bewley.kgrid    = kgrid;

bewley.Ntot     = bewley.knum*bewley.nx*bewley.ne;
Ntot            = bewley.Ntot;


%%% storing the solution of the model
bewley.Transg   = sparse(Ntot,Ntot);
bewley.Transyk  = sparse(Ntot,Ntot);
bewley.decis    = zeros(1,Ntot);
bewley.lambda   = zeros(Ntot,1);
bewley.U        = 0;

bewley.meanA    = 0;
bewley.meanL    = 0;
bewley.v        = zeros(Ntot,1);
bewley.r        = 0;
bewley.wage     = 0;
bewley.x        = 0.0;       % inefficiency of UI (for some specifications, useless otherwise)
bewley.tau      = (1/(1-bewley.x))*Urec*bewley.b/(1 - Urec);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    2)  Solving the steady-state model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ind_check    
    r              = 0.01;
    maxiter        = 50;
    maxV           = 20000;
    display        = true; 
    [bewley,distr] = solve_steady(bewley,r,maxiter,maxV,display);
    save simul_eco
end;


load  simul_eco              % file where the steady-state is
N        = 6; 
periods  = 0;                 % set 0 to have theoretical moments, number of periods to simulate otherwise
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    3)  Projecting and simulating with EXPLICIT partitions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
proj      = projection_explicit(bewley,N);
write_dynare_simul(bewley,proj,periods);
dynare Dynare_simul
  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    4)  Projecting and simulating with IMPLICT partitions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 proj      = projection_implicit(bewley,N);
 write_dynare_simul(bewley,proj,periods);
 dynare Dynare_simul
 
   
  

